<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Traits\TCrud;
use Exception;

class PerfilacessoLabel
{

    use TCrud;

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;
    /**
     * @var \ACL\Repositories\PerfilacessoLabel
     */
    private $perfilacessoLabelRepository;

    public function __construct()
    {
        $this->libEnvironment = InstanceProvider::getLibEnvironment();
        $this->perfilacessoLabelRepository = InstanceProvider::getRepository()->getPerfilacessoLabel();
    }

    /**
     * @param $nrperfilacesso
     * @return mixed
     * @throws Exception
     */
   public function findPerfilacessoLabel($nrperfilacesso)
   {
       $perfilacessoLabels = $this->perfilacessoLabelRepository->findPerfilacessoLabelByNrperfilacesso($nrperfilacesso, getNrorg());
       $labelsFile = $this->getFileAclLabelsJson();

       if (!$labelsFile) {
           throw new Exception('Arquivo "aclLabels.json" não encontrado');
       }

       $labelsFile = $this->setLabelsVisibility($perfilacessoLabels, $labelsFile);

       return $labelsFile;
   }

    /**
     * @return mixed
     * @throws Exception
     */
   private function getFileAclLabelsJson()
   {
       $productBasePath = InstanceProvider::getLibUtilities()->getProductBasePath();
       return json_decode(@file_get_contents($productBasePath . 'aclLabels.json'), true);
   }

    /**
     * @param $perfilacessoLabels
     * @param $labelsFile
     * @return mixed
     */
   private function setLabelsVisibility($perfilacessoLabels, &$labelsFile)
   {
       foreach ($labelsFile as &$label) {
           $label['data']['checked'] = false;
           foreach ($perfilacessoLabels as $row) {
               if ($label['data']['type'] !== 'GROUPED' && $row['LABEL'] == $label['data']['label']) {
                   $label['data']['checked'] = true;
               }

               if (array_key_exists('children', $label) && is_array($label['children']) && count($label['children'])) {
                   $this->setLabelsVisibility($perfilacessoLabels, $label['children']);
               }
           }
       }
       return $labelsFile;
   }

    /**
     * @param $labels
     * @param $nrperfilacesso
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function savePerfilacessoLabel($labels, $nrperfilacesso)
    {
        $labelsQueryIn = $this->getStringInQueryByArray($labels, 'LABEL');
        $perfilacessoLabels = $this->perfilacessoLabelRepository->findPerfilacessoLabelByNrperfilacessoAndLabels($nrperfilacesso, $labelsQueryIn, getNrorg());

        if (empty($perfilacessoLabels)) {
            foreach ($labels as $label) {
                $this->createPerfilacessoLabel($nrperfilacesso, $label);
            }
        } else {
            foreach ($labels as $label) {
                $key = in_array($label, array_column($perfilacessoLabels, 'LABEL'));
                if ($key === false) {
                    $this->createPerfilacessoLabel($nrperfilacesso, $label);
                }
            }
//            foreach ($perfilacessoLabels as $row) {
//                foreach ($labels as $label) {
//                    if ($row['LABEL'] !== $labels) {
//                        $this->createPerfilacessoLabel($nrperfilacesso, $label);
//                    }
//                }
//            }
        }
    }

    /**
     * @param array $labels
     * @param string $column
     * @param string $condition
     * @return string
     */
    private function getStringInQueryByArray(array $labels, string $column, string $condition = 'AND'): string
    {
        $queryIn = "$condition $column IN (";
        foreach ($labels as $label) {
            $queryIn .= "'" . $label . "', ";
        }
        $queryIn = substr($queryIn, 0, -2) . ")";
        return $queryIn;
    }

    /**
     * @param $nrperfilacesso
     * @param $label
     * @param string $idativo
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    private function createPerfilacessoLabel($nrperfilacesso, $label, $idativo = 'S')
    {
        $newCode = InstanceProvider::getNewCode()->getCodeByTableName('PERFILACESSO_LABEL', getNrorg(), 1, 10, true);
        $auditFields = $this->perfilacessoLabelRepository->getAuditFieldValuesDefaultInsert();
        $this->perfilacessoLabelRepository->savePerfilacessoLabel(
            $newCode, $label, intval($nrperfilacesso), $auditFields['NRORG'], $idativo, $auditFields['CDOPERINCLUSAO']);
    }

    /**
     * @param $labels
     * @param $nrperfilacesso
     * @throws \Doctrine\DBAL\Driver\Exception
     * @throws \Doctrine\DBAL\Exception
     */
    public function deletePerfilacessoLabel($labels, $nrperfilacesso)
    {
        $labelsQueryIn = $this->getStringInQueryByArray($labels, 'LABEL');
        $this->perfilacessoLabelRepository->deletePerfilacessoLabel($nrperfilacesso, $labelsQueryIn, getNrorg());
    }
}
